/* File Cell.C  implementation of class Cell */

/*		Copyright (c) 1990 SYCO			  */
/*		  All Rights Reserved			  */
/*							  */
/*  THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF SYCO   */
/*  The copyright notice above does not evidence any      */
/*  actual or intended publication of such source code.   */


#define CELL_C

#include <oak/oakstrn.h>
#include <oak/symbol.h>
#include <oak/cbacks.h>
#include <mondrian/widget.h>

#include "Cell.h"

#include "Lot.h"
#include "Piece.h"


#define THIS Cell
#define BASE Server




Symbol CellDoLot = "CellDoLot";
Symbol LotCompleted = "LotCompleted";

static int errorFlag = 0; 

#ifdef CELL_IS_MAIN  /* remove main */

#if 1

#include <mondrian/animatio.h>

main(int,char**)
{
  Cell* cell = new Cell;
  setRealTime(100);
  if(!searchOption("-noDebug"))
  {
    ANIMATE_ON_WIDGET(cell,"Cell");
    ANIMATE_CHILD(mac,"Machine-Stub");
    ANIMATE_CHILD(tr,"Transport");

    START_ANIMATION;

    Lot* lot = new Lot("Lotto000","Biella",3);
    cell->waitLot(lot);
    delete lot;  

    END_ANIMATION;
  }
  else
  {
    cell->start();
    Lot* lot = new Lot("Lotto000","Biella",3);
    cell->waitLot(lot);
    delete lot;  
    cell->stop();
  }
  delete cell;

  return 0;
}

#else

main(int,char**)
{
  Cell* cell = new Cell;

  cell->start();

  Lot* lot = new Lot("Lotto000","Biella",3);
  cell->waitLot(lot);
  delete lot;  

  cell->stop();
  delete cell;
  return 0;
}
#endif /* normal main or graphic main */
#endif /* remove main */




DEFINE_SIMPLE_CLASS(Cell,Server,1,NULL);


Cell::Cell()
  : BASE("Cell"),
    storeIn("StoreIn"), storeOut("StoreOut"),
    mac("Machine",3), tr("Transport",3)
{
}


Cell::Cell(Object* parentWidget)
  : BASE("Cell"),
    storeIn("StoreIn"), storeOut("StoreOut"),
    mac("Machine",3), tr(parentWidget,"Transport",3)
{
  TRACE_FUNC("Cell::Cell");
}




void Cell::fillStoreIn(Lot& lot)
{
  TRACE_FUNC("Cell::fillStoreIn");
  storeIn.resizePiece((size_t)lot.pieceNumber());
  storeOut.resizePiece((size_t)lot.pieceNumber());
  for(int i = 0; i < lot.pieceNumber(); i++)
	storeIn.addPiece(new Piece(
		form("piece%d",i),lot.pieceType()));

}


void Cell::emptyStoreOut()
{
  TRACE_FUNC("Cell::emptyStoreOut");
  storeOut.deleteAllPiece();
}



Object* Cell::doService(Service* service)
{
  TRACE_FUNC("Cell::doService");
  CBACK_TRACE(service,"Cell_doService|_START_");
  // Get Arguments
  GET_ARG(1,Lot,lot);
  fillStoreIn(lot);
  CBACK_TRACE(service,"Cell_doService|7427932");
  // Define Variables
  Service* trServ = NULL;
  Service* macServ = NULL;
  Piece* piece = NULL;
  Symbol* partProg = new Symbol("Fora1Biella");
  CBACK_TRACE(service,"Cell_doService|3904260");
  // Mission Req
  piece = storeIn.firstPiece();
  trServ = tr.joinMission(&storeIn,piece,&mac);
  ev.attachTo(trServ,WaitLoad);
  ev.attachTo(&storeIn,RemoveSignal);
  ev.attachTo(&storeOut,FullSignal);
  piece = (Piece*)nil;
  CBACK_TRACE(service,"Cell_doService|3808692");
  service->setStatus(WaitEvent);
  SELECT_TRACE(&ev,"Cell_doService|3812108")
    ACCEPT(MacOutFull);
      macServ = CASTP(Service,ev.owner());
      ev.detachFrom(macServ,MacOutFull);
      piece = CASTP(Piece,ev.callData());
      macServ->signal();
      CBACK_TRACE(service,"Cell_doService|4104364");
      // Mission Req
      trServ = tr.joinMission(&mac,piece,&storeOut);
      ev.attachTo(trServ,WaitLoad);
      piece = (Piece*)nil;
      CBACK_TRACE(service,"Cell_doService|4190580");

      CBACK_TRACE(service,"Cell_doService|1698692");
      service->setStatus(WaitEvent, NULL);
      CONTINUE;
    ACCEPT(WaitLoad);
      trServ = CASTP(Service,ev.owner());
      CBACK_TRACE(service,"Cell_doService|4261780");
      if( &storeIn != ev.callData() )
      {
          CBACK_TRACE(service,"Cell_doService|6917828");
          // Mac unload
          Machine* src = CASTP(Machine,ev.callData());
          piece = src->waitRemovePiece();
          CBACK_TRACE(service,"Cell_doService|8849028");
      }
      else
      {
          CBACK_TRACE(service,"Cell_doService|6917828");
          // StoreIn unload
          piece = storeIn.removePiece();
          CBACK_TRACE(service,"Cell_doService|6971148");
      }
      // Transp load
      ev.detachFrom(trServ,WaitLoad);
      ev.attachTo(trServ,WaitUnload);
      tr.waitAddPiece(piece);
      piece = (Piece*)nil;
      CBACK_TRACE(service,"Cell_doService|4315660");

      CBACK_TRACE(service,"Cell_doService|1336316");
      service->setStatus(WaitEvent, NULL);
      CONTINUE;
    ACCEPT(WaitUnload);
      trServ = CASTP(Service,ev.owner());
      CBACK_TRACE(service,"Cell_doService|4378804");
      // Transp unload
      ev.detachFrom(trServ,WaitUnload);
      tr.signal(trServ);
      piece = tr.waitRemovePiece(trServ);
      CBACK_TRACE(service,"Cell_doService|4238196");
      if( &storeOut == ev.callData() )
      {
          CBACK_TRACE(service,"Cell_doService|9255436");
          // StoreOut load
          storeOut.addPiece(piece);
          piece = (Piece*)nil;
          // if(errorFlag++ > 0)
          // cout << piece->type() << endl;
          CBACK_TRACE(service,"Cell_doService|9359788");
      }
      else
      {
          CBACK_TRACE(service,"Cell_doService|9255436");
          // Mac load
          Machine* dst = CASTP(Machine,ev.callData());
          macServ = dst->joinWork(piece,partProg);
          ev.attachTo(macServ,MacOutFull);
          piece = (Piece*)nil;
          CBACK_TRACE(service,"Cell_doService|6998372");
      }

      CBACK_TRACE(service,"Cell_doService|1718964");
      service->setStatus(WaitEvent, NULL);
      CONTINUE;
    ACCEPT(FullSignal);
      
      CBACK_TRACE(service,"Cell_doService|3275892");
      // Lot Complete
      ev.detachFrom(&storeOut,FullSignal);
      delete partProg;
      emptyStoreOut();
      service->setStatus(LotCompleted,&lot);
      CBACK_TRACE(service,"Cell_doService|2516692");
    ACCEPT(RemoveSignal);
      
      CBACK_TRACE(service,"Cell_doService|10060092");
      if( storeIn.numberOfPiece() == 0 )
      {
          CBACK_TRACE(service,"Cell_doService|10326788");
          // Stop Monitor
          ev.detachFrom(&storeIn,RemoveSignal);
          CBACK_TRACE(service,"Cell_doService|10435388");
      }
      else
      {
          CBACK_TRACE(service,"Cell_doService|10326788");
          // Mission Req
          piece = storeIn.firstPiece();
          trServ = tr.joinMission(&storeIn,piece,&mac);
          ev.attachTo(trServ,WaitLoad);
          piece = (Piece*)nil;
          CBACK_TRACE(service,"Cell_doService|8082076");
      }

      CBACK_TRACE(service,"Cell_doService|10375092");
      service->setStatus(WaitEvent, NULL);
      CONTINUE;
  END_SELECT

  CBACK_TRACE(service,"Cell_doService|3798732");
  return nil;
}



void Cell::waitLot(Lot* lot)
{
  TRACE_FUNC("Cell::waitLot");
  waitService(&CellDoLot,lot);
}


void Cell::start()
{
  TRACE_FUNC("Cell::start");
  tr.start();
}


void Cell::stop()
{
  TRACE_FUNC("Cell::stop");
  tr.stop();
}


void Cell::waitSetup()
{
  TRACE_FUNC("Cell::waitSetup");
  notYetImplemented("waitSetup");
}


void Cell::repair()
{
  TRACE_FUNC("Cell::repair");
  notYetImplemented("repair");
}


void Cell::maintain()
{
  TRACE_FUNC("Cell::maintain");
  notYetImplemented("maintain");
}




Object* Cell::editorTrace(String& s)
{
  if(s == "ev")
  {
	s = ev.printOnString();
	return (Object*)&ev;
  }
  else
  if(s == "storeOut")
  {
	s = storeOut.printOnString();
	return (Object*)&storeOut;
  }
  else
  if(s == "storeIn")
  {
	s = storeIn.printOnString();
	return (Object*)&storeIn;
  }
  else
  if(s == "tr")
  {
	s = tr.printOnString();
	return (Object*)&tr;
  }
  else
  if(s == "mac")
  {
	s = mac.printOnString();
	return (Object*)&mac;
  }
  else
  if(s == "configurationData")
  {
	s = configurationData.printOnString();
	return (Object*)&configurationData;
  }
  else
  if(s == "technologicalData")
  {
	s = technologicalData.printOnString();
	return (Object*)&technologicalData;
  }
  return nil;
}

/* End of File CELL */
