/* File ChangeRail.C  implement ....*/
#define CHANGERAIL_C


#include <oak/oakstrn.h>
#include <oak/symbol.h>
#include <oak/cbacks.h>

#include "ChangeRail.h"

#include "RailSemaph.h"
#include <mondrian/fillarc.h>


DEFINE_SIMPLE_CLASS(ChangeRail,TurnRail,1,NULL);
#define THIS ChangeRail
#define BASE TurnRail


void ChangeRail::storer(fileDescTy& fd) const
  { BASE::storer(fd); }

void ChangeRail::storer(ostream& strm) const
{
  TRACE_FUNC("ChangeRail::storer");
  BASE::storer(strm);
  strm << " " << rail_len1 << " " << turn_change 
       << " " ; // << from_rail << " ";
  sem1->storeOn(strm);
  sem2->storeOn(strm);
  sem3->storeOn(strm);
}

ChangeRail::ChangeRail(fileDescTy& fd, ChangeRail& where)
  :BASE(fd,where)  { THIS_EQUAL_WHERE(where); } 

ChangeRail::ChangeRail(istream& strm, ChangeRail& where)
  :BASE(strm,where)
{
  THIS_EQUAL_WHERE(where);
  strm >> rail_len1 >> turn_change; //  >> from_rail;
  sem1 = (RailSemaph*)readFrom(strm);
  sem2 = (RailSemaph*)readFrom(strm);
  sem3 = (RailSemaph*)readFrom(strm);
}


ChangeRail::ChangeRail(double ang, double ray, double len)
  :BASE(ang,ray), rail_len1(len), turn_change(YES)
{
  rails.reSize(3);
  posCltn.reSize(3);
  if(ang < 0)
	rail_len = -ray;
  sem1 = (RailSemaph*)nil;
  sem2 = (RailSemaph*)nil;
  sem3 = (RailSemaph*)nil;
}



ChangeRail::~ChangeRail()
{
  TRACE_FUNC("ChangeRail::~ChangeRail");
  if(sem1 != nil)
	sem1->removeChange(this);
  if(sem2 != nil)
	sem2->removeChange(this);
  if(sem3 != nil)
	sem3->removeChange(this);
}





void ChangeRail::setGeometry(const RailPos& pos, int orient)
{
  assert(orient > 0 && orient < 4);
  if(orient == 3)
  {
    posCltn[2] = new RailPos(-pos);
    RailPos pos1 = pos + RailPos(rail_len1,0,0); 
    BASE::setGeometry(-pos1,1);	
  }
  else
  {
    BASE::setGeometry(pos,orient);	
    posCltn[2] = new RailPos(-posAt(1) + RailPos(rail_len1,0,0));
  }
}


void ChangeRail::createIcon()
{
  TRACE_FUNC("ChangeRail::createIcon");

  BASE::createIcon();
  double x = posAt(1).x();
  double y = posAt(1).y();
  RailPos diff = posAt(3) - posAt(1);
  double w = diff.x();
  double h = diff.y();
  add(*new DrawLine(x,y,w,h));
}


RailPos ChangeRail::getConveyPos()
{
  TRACE_FUNC("ChangeRail::getConveyPos");
  if(next_rail == 2 || from_rail == 2)
	return BASE::getConveyPos();
  RailPos& p1 = posAt(1);
  RailPos& p2 = posAt(3);
  double x = (p1.x() + p2.x());
  double y = (p1.y() + p2.y());
  double ang = (-p1).angle() + p2.angle();
  return RailPos(x/2, y/2, ang/2);
}



bool ChangeRail::change()
{
  return turn_change;
}


void ChangeRail::setChange(bool change)
{
  turn_change = change;
  if(turn_change && next_rail == 3)
	next_rail = 2;
  else
  if(!turn_change && next_rail == 2)
	next_rail = 3;

}


bool ChangeRail::toggleChange()
{
  setChange(!turn_change);
  return turn_change;
}


int ChangeRail::setFrom(Rail* rail)
{
  TRACE_FUNC("ChangeRail::setFrom");
  from_rail = rails.indexOf(*rail) + 1;
  if(from_rail == 3)
    next_rail = 1;
  else
  if(!turn_change && from_rail == 1)
    next_rail = 3;
  else
    next_rail = 3 - from_rail;
  return from_rail;
}


FillArc* ChangeRail::addSemaph(RailSemaph* semaph, int where)
{
  TRACE_FUNC("ChangeRail::addSemaph");
  RailPos p;
  switch(where)
  {
	case 1:
	   sem1 = semaph;
  	   p = posAt(1) + RailPos(0, 0.4, 0);
	   break;
	case 2:
	   sem2 = semaph;
  	   p = posAt(2) + RailPos(0, 0.4, 0);
	   break;
	case 3:
	   sem3 = semaph;
  	   p = posAt(3) + RailPos(0, 0.4, 0);
	   break;
	default:
	   assert(0);
  }
  FillArc* light = new FillArc(p.x(),p.y(),0.3, 0.3,0,360);
  add(*light);
  return light;
}


RailSemaph* ChangeRail::fromSemaph()
{
  TRACE_FUNC("ChangeRail::fromSemaph");
  switch(from_rail)
  {
	case 1:
		return sem1;
	case 2:
		return sem2;
	case 3:
		return sem3;
	default:
		assert(0);
		return sem1;
  }
}


RailSemaph* ChangeRail::toSemaph()
{
  TRACE_FUNC("ChangeRail::toSemaph");
  switch(next_rail)
  {
	case 1:
		return sem1;
	case 2:
		return sem2;
	case 3:
		return sem3;
	default:
		assert(0);
		return sem1;
  }
}


void ChangeRail::removeSemaph(RailSemaph* semaph, FillArc* light)
{
  TRACE_FUNC("ChangeRail::removeSemaph");
  if(sem1 == semaph)
	sem1 = (RailSemaph*)nil;
  else
  if(sem2 == semaph)
	sem2 = (RailSemaph*)nil;
  else
  {
	assert(sem3 == semaph);
	sem3 = (RailSemaph*)nil;
  }
  remove(*light);
  delete light;
}


RailSemaph* ChangeRail::toSemaph(Rail* from)
{
  TRACE_FUNC("ChangeRail::toSemaph");
  int fromRail = rails.indexOf(*from) + 1;
  if(fromRail == 3 || fromRail == 2)
	return sem1;
  else
  if(!turn_change)
	return sem3;
  else
	return sem2;
}


size_t ChangeRail::railCapacity() const
{
  return 3;
}




Object* ChangeRail::editorTrace(String& s)
{
  if(s == "sem1")
  {
	s = sem1->printOnString();
	return (Object*)sem1;
  }
  else
  if(s == "sem3")
  {
	s = sem3->printOnString();
	return (Object*)sem3;
  }
  else
  if(s == "sem2")
  {
	s = sem2->printOnString();
	return (Object*)sem2;
  }
  return nil;
}

/* End of File ChangeRail.C */

