/* File Rail.C  implement ....*/
#define RAIL_C


#include <oak/oakstrn.h>
#include <oak/symbol.h>
#include <oak/cbacks.h>

#include "Rail.h"


#include <oak/orderedc.h>
DEFINE_SIMPLE_CLASS(Rail,CompositeIcon,1,NULL);
#define THIS Rail
#define BASE CompositeIcon


void Rail::storer(fileDescTy& fd) const
  { BASE::storer(fd); }

void Rail::storer(ostream& strm) const
{
  TRACE_FUNC("Rail::storer");
  OrderedCltn cltn;
  OrderedCltn* thisCltn = (OrderedCltn*)iconCltn;
  DO(*thisCltn,DrawIcon,icon)
	cltn.add(*icon);
  OD
  thisCltn->removeAll();
  BASE::storer(strm);
  posCltn.storeOn(strm);
  rails.storeOn(strm);
  // conveyCltn.storeOn(strm);
  // strm << " " << next_rail 
  strm << " " << rail_len
       << " " << sz << " ";
  DO(cltn,DrawIcon,icon)
	thisCltn->add(*icon);
  OD
}

Rail::Rail(fileDescTy& fd, Rail& where)
  :BASE(fd,where)  { THIS_EQUAL_WHERE(where); } 

Rail::Rail(istream& strm, Rail& where)
  :BASE(strm,where)
{
  THIS_EQUAL_WHERE(where);
  readFrom(strm,"ArrayOb",posCltn);
  readFrom(strm,"ArrayOb",rails);
  // readFrom(strm,"ArrayOb",conveyCltn);
  // strm >> next_rail
  strm >> rail_len >> sz;

  next_rail = 2;
  from_rail = 1; 
}


Rail::Rail(double len)
  : rail_len(len),rails(2), posCltn(2)
{
  sz = 0;
  next_rail = 2;
  from_rail = 1;
}



Rail::~Rail()
{
  TRACE_FUNC("Rail::~Rail");
}





void Rail::setGeometry(const RailPos& pos, int orient)
{
  assert(orient > 0 && orient < 3);
  posCltn[orient - 1] = new RailPos(-pos);
  posCltn[2 - orient] = new 
	RailPos(pos + RailPos(rail_len,0,0)); 
}


void Rail::setConnection(Rail* rail, int orient)
{
  assert(rails[orient-1] == nil);
  rails[orient-1] = rail;
  sz++;
}


RailPos Rail::getConveyPos()
{
  RailPos& p1 = posAt(1);
  RailPos& p2 = posAt(2);
  double x = (p1.x() + p2.x());
  double y = (p1.y() + p2.y());
  double ang = (-p1).angle() + p2.angle();
  return RailPos(x/2, y/2, ang/2);
}


void Rail::createMarker(const RailPos& pos)
{
  double dx = 0.1 * sin(pos.radAngle());
  double dy = 0.1 * cos(pos.radAngle());
  add(*new DrawLine(pos.x() + dx, pos.y() - dy,
		-2 * dx, 2 * dy));
}


void Rail::removeConnect(Rail* rail)
{
  rails.remove(*rail);
  sz--;
}


void Rail::createIcon()
{
  TRACE_FUNC("Rail::createIcon");

  RailPos pos(posAt(1));
  RailPos diff = posAt(2) - pos;

  add(*new DrawLine(pos.x(),pos.y(),diff.x(),diff.y()));
}



Rail* Rail::setPos(const RailPos& pos, int orient)
{
  if(posCltn[orient - 1] == nil)
  {
    	setGeometry(pos,1);
	createIcon();
  }
  else
	cerr << "Position already assign for " 
	     << *this << endl;
  return this;
}


Rail* Rail::attachIn(int thisOrient, Rail* rail, int itsOrient)
{
  TRACE_FUNC("Rail::attachIn");
  rail->setConnection(this,itsOrient);
  setConnection(rail,thisOrient);
  if(rail->posCltn[itsOrient - 1] == nil)
  {
  	rail->setGeometry(posAt(thisOrient),itsOrient);
	rail->createIcon();
    	createMarker(posAt(thisOrient));
  }
  else
  {
	createMarker(posAt(thisOrient));
	cout << rail->posAt(itsOrient) -
		posAt(thisOrient) << endl;
  }

  return rail;
}


Rail* Rail::detachAt(int orient, int& attachOrient)
{
  if(rails[orient - 1] == nil)
	return (Rail*)nil;

  Rail* rail = CASTP(Rail,rails[orient - 1]);
  removeConnect(rail);
  for(int i = rail->railCapacity(); i-- > 0;)
  {
	if(rail->rails.atIndex(i) != this)
		continue;
	attachOrient = i + 1; 
      rail->removeConnect(this);
      return rail;
  }
  assert(0);
  return rail;
}


Rail* Rail::railAt(int orient) const
{
  return CASTP(Rail,rails.atIndex(orient - 1));
}


RailPos& Rail::posAt(int orient) const
{
  return *CASTP(RailPos,posCltn.atIndex(orient - 1));
}


Rail* Rail::nextRail(Rail* from)
{
  if(nil != from)
	setFrom(from);

  if(rails.atIndex(next_rail - 1) == nil)
  {
    DO(rails,Object,r)
      if(r != nil)
	  return CASTP(Rail,r);
    OD
  }
  return railAt(next_rail);
}


int Rail::setFrom(Rail* rail)
{
  TRACE_FUNC("Rail::setFrom");
  from_rail = rails.indexOf(*rail) + 1;
  next_rail = 3 - from_rail;
  return from_rail;
}


RailPos Rail::getCartPos()
{
  TRACE_FUNC("Rail::getCartPos");
  return getConveyPos();
}


RailPos Rail::getStationPos()
{
  TRACE_FUNC("Rail::getStationPos");
  return getCartPos() + RailPos(0.0,1.5);
}


void Rail::initAfterLoad()
{
  TRACE_FUNC("Rail::initAfterLoad");
  if(posCltn[0] != nil)
	createIcon();
}


void Rail::printOn(ostream& strm) const
{
  strm << className() << "[\n";
  for(int i = 0; i < posCltn.capacity(); i++)
  {
	if(posCltn.atIndex(i) == nil)
		break;
	strm << "\t" << *posCltn.atIndex(i)
	     << " > " << rails.atIndex(i)->className() nl;
  }
  strm << "]" nl;
  BASE::printOn(strm);
}


size_t Rail::railCapacity() const
{
  return 2;
}


size_t Rail::railSize() const
{
  return sz;
}




Object* Rail::editorTrace(String& s)
{
  if(s == "rails")
  {
	s = rails.printOnString();
	return (Object*)&rails;
  }
  else
  if(s == "posCltn")
  {
	s = posCltn.printOnString();
	return (Object*)&posCltn;
  }
  return nil;
}

/* End of File Rail.C */

