/* File RemoteContext.C  implement ....*/
#define REMOTECONTEXT_C


#include <oak/oakstrn.h>
#include <oak/symbol.h>
#include <oak/cbacks.h>

#include "RemoteContext.h"


#include <oak/orderedc.h>
#include <oak/lookupke.h>

Symbol ContextWaitEvent("ContextWaitEvent");

#ifdef GRAPH
# define CLS_FILE "RemoteContext"
# define SERVER_CLS "Server"
# include <mondrian/toplevel.h>
# include <mondrian/form.h>

# include <classEdt/animobj.h>
# include <classEdt/controlpanel.h>

static bool changeAnim(const Cback_struct& cb)
{
  CASTP(ControlPanel,cb.userData())->waitStep();
  return NO;
}
#endif

void setMainStreamProcessName(int,char**)
{
  mainStreamProcessName = "RemoteContext";
  if(!searchOption("-procId",mainStreamProcessName))
	cerr << "Warning: any procId option found" << endl;
}

main(int , char**)
{
  char* stubId = "StubProcess";
  if(!searchOption("-stubId",stubId))
	cerr << "Warning: any stubId option found" << endl;

#ifdef GRAPH
  RemoteMachine* context = 
	new RemoteMachine(mainStreamProcessName,stubId);

  CppForm* contextForm = new CppForm(topLevel,"ContextForm");
  ControlPanel* ctrlPanel = new ControlPanel(contextForm);
  AnimateObject& anim = *new AnimateObject(contextForm,context,CLS_FILE);
  anim.animateInclude("server",SERVER_CLS);
  anim.addCback(ChangeAnimator,changeAnim,ctrlPanel);
  contextForm->topBottom(ctrlPanel,anim);
  realizeTopLevel();
  ctrlPanel->waitStep();

  context->waitService();
  ctrlPanel->waitEnd();  
  delete context;
#else
  RemoteMachine* context = 
	new RemoteMachine(mainStreamProcessName,stubId);
  context->waitService();
  delete context;
#endif  
  return 0;
}

DEFINE_SIMPLE_CLASS(RemoteContext,Server,1,NULL);
#define THIS RemoteContext
#define BASE Server


RemoteContext::RemoteContext(const char* name, const char* stubId)
  :BASE(name), network(stubId)
{
}



RemoteContext::~RemoteContext()
{
  yield();
  network.closeConnection();
  // assert(!ev.someAttach()); // forgot some detach
  // assert(size() == 0); // no more service run
  // assert(server.size() == 0);
}



void RemoteContext::otherMessage(OrderedCltn& message)
{
  cout << message << endl;
}


void RemoteContext::runSubService(OrderedCltn& message)
{
  Integer& serviceId = *CASTP(Integer,message[1]->copy());
  // prepare argument for subService 
  Service* subService = server.joinService();
  services.addAssoc(*subService,serviceId);
  idDict.addAssoc(serviceId,*subService);
  ev.attachTo(subService,AnyCback);
}


void RemoteContext::sendCbackMessage(Service& subServ, const Symbol& event, Object* callData)
{
  Object* servId = services.atKey(subServ);
  // use calldata for send custom parameter
  network.sendCltnMsg("Callback",&event,servId);
}


void RemoteContext::endSubService(Service& subServ)
{
  ev.detachFrom(&subServ,AnyCback);
  LookupKey* ass = services.removeKey(subServ);
  Object* servId = ass->value();
  delete ass;
  ass = idDict.removeKey(*servId);
  delete ass;
  delete servId;
  subServ.signal();
}



Object* RemoteContext::doService(Service* service)
{

  CBACK_TRACE(service,"RemoteContext_doService|_START_");
  
  ev.attachTo(&network,MessageReceived);
  ev.attachTo(&network,ConnectionClosed);

  CBACK_TRACE(service,"RemoteContext_doService|537925252");
  SELECT_STATUS_TRACE(&ev,ContextWaitEvent,NULL,"RemoteContext_doService|538064644")
    ACCEPT(MessageReceived);

      CBACK_TRACE(service,"RemoteContext_doService|538096132");
      Object* data = ev.callData();

      CBACK_TRACE(service,"RemoteContext_doService|538089860");
      if( validObject(data) && data->isKindOf(class_OrderedCltn) )
      {
      }
      else
      {
          continue;
      }
      OrderedCltn& message = *CASTP(OrderedCltn,data);
      String msgId = CASTP(String,message[0])->left(4);

      CBACK_TRACE(service,"RemoteContext_doService|538171268");
      if( msgId == "exec" )
      {
          
          runSubService(message);

          CBACK_TRACE(service,"RemoteContext_doService|538202116");
      }
      else
      {
          
          otherMessage(message);

          CBACK_TRACE(service,"RemoteContext_doService|538326020");
      }
      message.deleteAll();
      delete data;

      CBACK_TRACE(service,"RemoteContext_doService|538342916");
      continue;
    ACCEPT(ConnectionClosed);

      CBACK_TRACE(service,"RemoteContext_doService|538132868");
    ELSE_SELECT

      CBACK_TRACE(service,"RemoteContext_doService|538089988");
      if( ev.event() == ServiceRunning )
      {
          continue;
      }
      else
      {
      }
      
      Service* subServ = CASTP(Service,ev.owner());
      sendCbackMessage(*subServ,ev.event(),ev.callData());

      CBACK_TRACE(service,"RemoteContext_doService|538484228");
      if( ev.event() == ServiceTerminated )
      {
          
          endSubService(*subServ);

          CBACK_TRACE(service,"RemoteContext_doService|538513924");
      }
      else
      {
      }
      continue;
  END_SELECT
  
  ev.detachFrom(&network,ConnectionClosed);
  ev.detachFrom(&network,MessageReceived);

  CBACK_TRACE(service,"RemoteContext_doService|538364932");

  CBACK_TRACE(service,"RemoteContext_doService|538071684");
  return nil;
}






Object* RemoteContext::editorTrace(String& s)
{
  if(s == "network")
  {
	s = network.printOnString();
	return (Object*)&network;
  }
  else
  if(s == "ev")
  {
	s = ev.printOnString();
	return (Object*)&ev;
  }
  else
  if(s == "server")
  {
	s = server.printOnString();
	return (Object*)&server;
  }
  else
  if(s == "idDict")
  {
	s = idDict.printOnString();
	return (Object*)&idDict;
  }
  else
  if(s == "services")
  {
	s = services.printOnString();
	return (Object*)&services;
  }
  return nil;
}

/* End of File RemoteContext.C */

