/* File RemoteMachine.C  implementation of class RemoteMachine */

/*		Copyright (c) 1990 SYCO			  */
/*		  All Rights Reserved			  */
/*							  */
/*  THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF SYCO   */
/*  The copyright notice above does not evidence any      */
/*  actual or intended publication of such source code.   */


#define REMOTEMACHINE_C

#include <oak/oakstrn.h>
#include <oak/symbol.h>
#include <oak/cbacks.h>
#include <mondrian/widget.h>

#include "RemoteMachine.h"



#define THIS RemoteMachine
#define BASE Server




#include <oak/orderedc.h>
#include <oak/lookupke.h>

Symbol ContextWaitEvent("ContextWaitEvent");

#include <mondrian/animatio.h>
#include <mondrian/command.h>

# define CLS_FILE "RemoteMachine"
# define SERVER_CLS "Machine"

main(int , char**)
{
  traceBool = NO;  // avoid trace FACILITY
  setRealTime(100);
  RemoteMachine* context = new RemoteMachine;

  if(!searchOption("-noDebug"))
  {
    ANIMATE(context,CLS_FILE);
    ANIMATE_CHILD(server,SERVER_CLS);
    START_ANIMATION;

    context->waitService();

    END_ANIMATION;
  }
  else
  {
    CppCommand* command = new CppCommand(topLevelWidget,"RemoteMachine");
    command->addBreak();
    context->waitService();
    command->label("END");
    baseMainLoop();
  }
  delete context;
  return 0;
}



DEFINE_SIMPLE_CLASS(RemoteMachine,Server,1,NULL);


RemoteMachine::RemoteMachine(const char* name)
  :BASE(name), server("Machine",3)
{
}



RemoteMachine::~RemoteMachine()
{
  yield();
  network.closeConnection();
  // assert(!ev.someAttach()); // forgot some detach
  // assert(size() == 0); // no more service run
  // assert(server.size() == 0);
}



void RemoteMachine::otherMessage(OrderedCltn& message)
{
  cerr << "Receive other message " << message << endl;
}


void RemoteMachine::runSubService(OrderedCltn& message)
{
  cerr << "runSubService " << message << endl;

  Symbol serviceKind = String((*CASTP(String,message[0]))(4));
  Integer& serviceId = *CASTP(Integer,message[1]->copy());

  Service* subService = NULL;
  if(serviceKind == MacDoWork)
  { 
    String& pieceName = *CASTP(String,message[2]);
    String& pieceType = *CASTP(String,message[3]);
    Piece* piece = new Piece(pieceName,pieceType);
    Symbol* partProgram = new Symbol(*CASTP(Symbol,message[4]));
    subService = server.joinService(&MacDoWork,piece,partProgram);
    pieces.addAssoc(*piece,*partProgram);
  }
  else
  {
    assert(serviceKind == MacRemovePiece);
    subService = server.joinService(&MacRemovePiece);
  }

  services.addAssoc(*subService,serviceId);
  idDict.addAssoc(serviceId,*subService);
  ev.attachTo(subService,AnyCback);
}


void RemoteMachine::sendCbackMessage(Service& subServ, const Symbol& event, Object* callData)
{
  Object* servId = services.atKey(subServ);
  // use calldata for send custom parameter
  String eventName = event;
  if(eventName.inString("Machine_do") < 0)
  	network.sendCltnMsg("Callback",(Object*)&event,servId);
}


void RemoteMachine::endSubService(Service& subServ)
{
  ev.detachFrom(&subServ,AnyCback);
  LookupKey* ass = services.removeKey(subServ);
  Object* servId = ass->value();
  delete ass;
  ass = idDict.removeKey(*servId);
  delete ass;
  delete servId;
  Symbol* servKind = CASTP(Symbol,subServ.getArg(0));
  if(*servKind == MacDoWork)
  {
  	subServ.join();
  }
  else
  {
    assert(*servKind == MacRemovePiece);
    Piece* piece = CASTP(Piece,subServ.join());
    LookupKey* ass = pieces.removeKey(*piece);
    Symbol* partProgram = CASTP(Symbol,ass->value());
    delete ass;
    delete piece;
    delete partProgram;
    cerr << "End piece get" << endl;
  }
}



Object* RemoteMachine::doService(Service* service)
{
  TRACE_FUNC("RemoteMachine::doService");
  CBACK_TRACE(service,"RemoteMachine_doService|_START_");
  
  ev.attachTo(&network,MessageReceived);
  ev.attachTo(&network,ConnectionClosed);
  CBACK_TRACE(service,"RemoteMachine_doService|537925252");
  service->setStatus(ContextWaitEvent);
  SELECT_TRACE(&ev,"RemoteMachine_doService|538064644")
    ACCEPT(MessageReceived);
      
      CBACK_TRACE(service,"RemoteMachine_doService|538096132");
      
      Object* data = ev.callData();
      CBACK_TRACE(service,"RemoteMachine_doService|538089860");
      if( validObject(data) && data->isKindOf(class_OrderedCltn) )
      {
          CBACK_TRACE(service,"RemoteMachine_doService|538132996");
      }
      else
      {
          CBACK_TRACE(service,"RemoteMachine_doService|538132996");

          CBACK_TRACE(service,"RemoteMachine_doService|538170116");
          continue;
      }
      
      OrderedCltn& message = *CASTP(OrderedCltn,data);
      String msgId = CASTP(String,message[0])->left(4);
      CBACK_TRACE(service,"RemoteMachine_doService|538171268");
      if( msgId == "exec" )
      {
          CBACK_TRACE(service,"RemoteMachine_doService|538195332");
          
          runSubService(message);
          CBACK_TRACE(service,"RemoteMachine_doService|538202116");
      }
      else
      {
          CBACK_TRACE(service,"RemoteMachine_doService|538195332");
          
          otherMessage(message);
          CBACK_TRACE(service,"RemoteMachine_doService|538326020");
      }
      
      message.deleteAll();
      delete data;
      CBACK_TRACE(service,"RemoteMachine_doService|538342916");

      CBACK_TRACE(service,"RemoteMachine_doService|538300548");
      continue;
    ACCEPT(ConnectionClosed);
      
      CBACK_TRACE(service,"RemoteMachine_doService|538132868");
    ELSE_SELECT
      
      CBACK_TRACE(service,"RemoteMachine_doService|538089988");
      if( ev.event() == ServiceRunning )
      {
          CBACK_TRACE(service,"RemoteMachine_doService|538480900");

          CBACK_TRACE(service,"RemoteMachine_doService|538478980");
          continue;
      }
      else
      {
          CBACK_TRACE(service,"RemoteMachine_doService|538480900");
      }
      
      Service* subServ = CASTP(Service,ev.owner());
      sendCbackMessage(*subServ,ev.event(),ev.callData());
      CBACK_TRACE(service,"RemoteMachine_doService|538484228");
      if( ev.event() == ServiceTerminated )
      {
          CBACK_TRACE(service,"RemoteMachine_doService|538584836");
          
          endSubService(*subServ);
          CBACK_TRACE(service,"RemoteMachine_doService|538513924");
      }
      else
      {
          CBACK_TRACE(service,"RemoteMachine_doService|538584836");
      }

      CBACK_TRACE(service,"RemoteMachine_doService|538618628");
      continue;
  END_SELECT
  
  ev.detachFrom(&network,ConnectionClosed);
  ev.detachFrom(&network,MessageReceived);
  CBACK_TRACE(service,"RemoteMachine_doService|538364932");

  CBACK_TRACE(service,"RemoteMachine_doService|538071684");
  return nil;
}





Object* RemoteMachine::editorTrace(String& s)
{
  if(s == "server")
  {
	s = server.printOnString();
	return (Object*)&server;
  }
  else
  if(s == "network")
  {
	s = network.printOnString();
	return (Object*)&network;
  }
  else
  if(s == "ev")
  {
	s = ev.printOnString();
	return (Object*)&ev;
  }
  else
  if(s == "services")
  {
	s = services.printOnString();
	return (Object*)&services;
  }
  else
  if(s == "idDict")
  {
	s = idDict.printOnString();
	return (Object*)&idDict;
  }
  else
  if(s == "pieces")
  {
	s = pieces.printOnString();
	return (Object*)&pieces;
  }
  return nil;
}

/* End of File REMOTEMACHINE */
