/* File Tcpip.C  implementation of class Tcpip */

/*		Copyright (c) 1990 SYCO			  */
/*		  All Rights Reserved			  */
/*							  */
/*  THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF SYCO   */
/*  The copyright notice above does not evidence any      */
/*  actual or intended publication of such source code.   */


#define TCPIP_C

#include <oak/oakstrn.h>
#include <oak/symbol.h>
#include <oak/cbacks.h>

#include "Tcpip.h"




#if defined(LYNXOS)
# define socket 		socketNoPar
#endif

#include <sys/socket.h>

#if defined(LYNXOS)
# undef socket
  CFUNC_BEGIN
  int socket(int,int,int);
  CFUNC_END
#endif

#if defined(ultrix) | (defined(IBM_RISC) & defined(OLD_CC_3_2)) | defined(LYNXOS)

# define gethostbyname 		gethostbynameNoPar
# define getservbyname 		getservbynameNoPar

# ifndef IBM_RISC
#  define ntohs 			ntohsNoPar
#  define htons 			htonsNoPar
# endif

#endif

#include <netinet/in.h>
#include <netdb.h>

#if defined(ultrix) | (defined(IBM_RISC) & defined(OLD_CC_3_2)) | defined(LYNXOS)
# undef gethostbyname
# undef getservbyname

# ifndef IBM_RISC
#  undef ntohs
#  undef htons
# endif

  CFUNC_BEGIN
	hostent* gethostbyname(const char*);
	servent* getservbyname(const char*, const char*);
#ifndef IBM_RISC
	unsigned short ntohs(int);
	int htons(unsigned short);
#endif
  CFUNC_END
#endif

CFUNC_BEGIN

  size_t inet_addr(const char* dotNotation);
  int shutdown(int,int);

#if defined(ultrix) | (defined(IBM_RISC) & defined(OLD_CC_3_2)) | defined(LYNXOS)
  int setsockopt(int,int,int,void*, int);
#elif !defined(IBM_RISC)
  int gethostname(char*,size_t);
#endif

#if !defined(hpux)
  size_t inet_network(const char* dotNotation);
  struct in_addr inet_makeaddr(size_t netNumb, size_t locAddr);
  const char* inet_ntoa(struct in_addr);
#else
# define IPPORT_USERRESERVED 5000
#endif

#if defined(IBM_RISC) &  defined(OLD_CC_3_2)
  int socket(int,int,int);
  int bind(int,sockaddr*,size_t);
  int connect(int,sockaddr*,size_t);
  int listen(int,int);
  int accept(int,sockaddr*,int*);
  int gethostname(char*,size_t);
#endif

CFUNC_END


#define NO_MORE_IO 2

#if 0

main(int,char**)
{
  Tcpip* network = new Tcpip;
  network->doIt();
  delete network;
  return 0;
}
#endif


DEFINE_SIMPLE_CLASS(Tcpip,Object,1,NULL);
#define THIS Tcpip
#define BASE Object


Tcpip::Tcpip()
{
  TRACE_FUNC("Tcpip::Tcpip");
}







void Tcpip::printError(const char* str)
{
  TRACE_FUNC("Tcpip::printError");
  if(errno != 0)
	perror(form("%s %d",str,errno));
  errno = 0;
}


int Tcpip::doConnect(const String& host, int portId)
{
  TRACE_FUNC("Tcpip::doConnect");

  hostent* remote_host = gethostbyname(host());
  if(remote_host == NULL)
  {
	  printError("gethost");
	  OAKexit(1);
  }

  sockaddr_in peer_addr;
  peer_addr.sin_family = AF_INET;
  peer_addr.sin_port = htons(portId);

  memcpy(&peer_addr.sin_addr, remote_host->h_addr, remote_host->h_length);

#if 0
  cout << form(" calling peer on %s (%d.%d.%d.%d) port %d\n",
             remote_host->h_name,
             ((unsigned char *)remote_host->h_addr)[0],
             ((unsigned char *)remote_host->h_addr)[1],
             ((unsigned char *)remote_host->h_addr)[2],
             ((unsigned char *)remote_host->h_addr)[3],
             ntohs(peer_addr.sin_port) ) << endl;

#endif

  int socketId = socket(AF_INET,SOCK_STREAM,IPPROTO_TCP);
  if(socketId == -1)
  {
	printError("socket");
	OAKexit(1);
  }
  else
	errno = 0;
  // cout << "Socket id " << socketId << endl;
  
  if(::connect(socketId,(sockaddr*)&peer_addr,sizeof(sockaddr_in)) == -1)
  {
	printError("connect");
	OAKexit(1);
  }

  printError("connect");
  
  char hostName[100];
  int ret = gethostname(hostName,100);
  int wr = write(socketId,hostName,strlen(hostName)+1);
  cout << "Sended = " << wr << endl;
  printError("write");
  return socketId;
}


int Tcpip::doListen(int portId)
{
  TRACE_FUNC("Tcpip::doListen");
  sockaddr_in myaddr;
  myaddr.sin_family = AF_INET;
  myaddr.sin_port = htons(portId) ;
  myaddr.sin_addr.s_addr = INADDR_ANY ;
  int listen_s = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
  if(listen_s == -1)
  {
	printError("socket");
	OAKexit(1);
  }
  else
	errno = 0;
  printError("socket");
  int reuseaddr_flag = 1 ;

  setsockopt(listen_s, SOL_SOCKET, SO_REUSEADDR,
              (char*)&reuseaddr_flag, sizeof(int));

  printError("setsock");


  if(bind(listen_s, (sockaddr*)&myaddr, sizeof(myaddr)) == -1)
  {
	printError("bind");
	OAKexit(1);
  }

  if(listen(listen_s, 5) == -1)
  {
	printError("listen");
	OAKexit(1);
  }
  return listen_s;
}


int Tcpip::doAccept(int listen_s, String& host)
{
  TRACE_FUNC("Tcpip::doAccept");
  int peeraddr_len = sizeof(sockaddr_in);
  sockaddr_in peeraddr;
  int channel = ::accept(listen_s, (sockaddr*)&peeraddr, &peeraddr_len);

  close (listen_s);
  printError("close listen");

  char hostName[100]; hostName[0] = 0;
  int rd = read(channel,hostName,100);
  cout << "Received = " << rd << " " << hostName << endl;
  printError("read");
  host = hostName;
  return channel;
}



void Tcpip::doIt()
{
  TRACE_FUNC("Tcpip::doIt");

  char* port = "0";
  searchOption("-port",port);
  int portId = (int)String(port).value();

  char* host = "localhost";
  searchOption("-host",host);
  if(!searchOption("-connect"))
  {
	String host;
	Pier channel = accept(portId,host);
  	char msg[200]; msg[0] = 0;
  	int rd = read(channel.in,msg,200);
  	cout << "Received = " << rd << " " << msg << endl;
  	printError("read");
  	close(channel.in);
	close(channel.out);
  	printError("close");
  }
  else
  {
	Pier channel = connect(host,portId);
	char* msg = "Messaggio";
	int wr = write(channel.out,msg,strlen(msg)+1);
	cout << "Sended = " << wr << endl;
	printError("write");
	close(channel.out);
	close(channel.in);
  	printError("close");
  }
}


Pier Tcpip::connect(const String& node, int port)
{
  TRACE_FUNC("Tcpip::connect");

  int portId1 = IPPORT_USERRESERVED + port * 2;
  int portId2 = portId1 + 1;
  Pier pier;
  int listen_s = doListen(portId2);
  pier.out = doConnect(node,portId1);
  String host;
  pier.in = doAccept(listen_s,host);
  return pier;
}


Pier Tcpip::accept(int port, String& host)
{
  TRACE_FUNC("Tcpip::accept");

  int portId1 = IPPORT_USERRESERVED + port * 2;
  int portId2 = portId1 + 1;

  int listen_s = doListen(portId1);

  Pier pier;
  pier.in = doAccept(listen_s,host);
  pier.out = doConnect(host,portId2);
  return pier;
}




Object* Tcpip::editorTrace(String& )
{
  return nil;
}

/* End of File TCPIP */

