/* File AgvSem.C  implementation of class AgvSem */

/*		Copyright (c) 1990 SYCO			  */
/*		  All Rights Reserved			  */
/*							  */
/*  THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF SYCO   */
/*  The copyright notice above does not evidence any      */
/*  actual or intended publication of such source code.   */


#define AGVSEM_C

#include <oak/oakstrn.h>
#include <oak/symbol.h>
#include <oak/cbacks.h>
#include <mondrian/widget.h>

#include "AgvSem.h"

#include "RailPos.h"
#include "TurnIcon.h"
#include "AgvTitle.h"
#include "RailIcon.h"
#include "Cart.h"
#include "Station.h"
#include "ChangeIcon.h"
#include "ChangeRail.h"


#define THIS AgvSem
#define BASE Semantic




#include <mondrian/toplevel.h>
#include <editor/graphedt.h>
#include <editor/graphsho.h>
#include <editor/textshow.h>

#include <oak/cbacks.h>
#include <oak/symbol.h>
#include <oak/exceptio.h>
#include <oak/oakstrm.h>
#include <oak/assoc.h>
#include <oak/stack.h>
#include <oak/integer.h>
#include <oak/arrayob.h>


#ifdef IBM_RISC
static void dummyFunc()
{
  new AgvSem;
  new RailIcon;
  new TurnIcon;
  new ChangeIcon;
  new CartIcon;
  new StationIcon;
}
#endif

#if 0

static bool closeFunc(const Cback_struct& )
{
  exitMainLoop();
  return NO;
}


main(int,char**)
{

  if(getArgc() > 2)
  {
        cerr << "Syntax error usage agvEdt <filename>" nl;
        OAKexit(1);
  }

  char* file = "noname";
  if(getArgc() > 1)
        file = getArgv()[1];

  Semantic* sem = CASTP(Semantic,class_AgvSem.create());
  String ext = sem->fileExtension();
  String fileName = file;
  if(fileName.right(ext.size()) != ext)
        fileName += ext;
  delete sem;

  setEditorSession();
  GraphEditor* editor = NULL;
  if( testWriteAndSet(fileName))
        editor = new GraphEditor(topLevel,class_AgvSem,file);
  else
  {
        cerr << "file read only" << endl;
        editor = new GraphShower(topLevel,class_AgvSem,file);
  }
  editor->addCback(CloseEditor,closeFunc);
  bool again = YES;
  while(again)
  {
        RaiseException x(OAK_ANY);
        BEGINX
                cppMainLoop();
                again = NO;
        OTHER_EXCEPTIONS
                cerr << EXCEPTION_DESCR << endl;
                editor->errorMessage(EXCEPTION_DESCR);
                exitMainLoop();
        OTHER_ENDX
  }
  if(editor->isMemberOf(class_GraphEditor))
        resetWrite(editor->fullName());
  delete editor;
  exitMainLoop();
  return 0;
}
#endif


DEFINE_SIMPLE_CLASS(AgvSem,Semantic,1,NULL);

void AgvSem::storer(fileDescTy& fd) const
  { BASE::storer(fd); }

void AgvSem::storer(ostream& strm) const
{
  TRACE_FUNC("AgvSem::storer");
  BASE::storer(strm);
  rails.storeOn(strm);
  carts.storeOn(strm);
  stations.storeOn(strm);
  freePoints.storeOn(strm);
  pointToRail.storeOn(strm);
  railToIcon.storeOn(strm);
  title->storeOn(strm);
}

AgvSem::AgvSem(fileDescTy& fd, AgvSem& where)
  :BASE(fd,where)  { THIS_EQUAL_WHERE(where); } 

AgvSem::AgvSem(istream& strm, AgvSem& where)
  :BASE(strm,where)
{
  THIS_EQUAL_WHERE(where);
  readFrom(strm,"OrderedCltn",rails);
  readFrom(strm,"OrderedCltn",carts);
  readFrom(strm,"OrderedCltn",stations);
  readFrom(strm,"OrderedCltn",freePoints);
  readFrom(strm,"Dictionary",pointToRail);
  readFrom(strm,"Dictionary",railToIcon);
  title = CASTP(AgvTitle,readFrom(strm));  
}


AgvSem::AgvSem()
{
  width(100);
  height(100);
  title = (AgvTitle*)nil;
}




AgvSem::~AgvSem()
{
  title = (AgvTitle*)nil;
}





const char* AgvSem::fileExtension() const
{
  return ".agv";
}


OrderedCltn& AgvSem::initGraph(OrderedCltn& cltn)
{
  title = new AgvTitle();
  title->x(w/2); title->y(3);
  title->titleName() = editor->getOnlyName().toUpper();
  cltn.add(*title);
  return cltn;
}


OrderedCltn& AgvSem::getIcons(OrderedCltn& cltn)
{
  cltn.add( *new String("LineRail") );
  cltn.add( *new String("TurnRail") );
  cltn.add( *new String("ChangeRail") );
  cltn.add( *new String("") );
  cltn.add( *new String("CartIcon") );
  cltn.add( *new String("StationIcon") );

  return cltn;
}


void AgvSem::setDrawSize()
{
  if(editor->isKindOf(class_GraphShower))
  {
        int drawWid = readInt("agvShowerDrawWidth",500);
        int drawHei = readInt("agvShowerDrawHeight",450);
        int minWid = readInt("agvShowerMinWidth",300);
        int minHei = readInt("agvShowerMinHeight",250);
        editor->setDrawSize(drawWid,drawHei,minWid,minHei);
  }
  else
  {
        int drawWid = readInt("agvEditorDrawWidth",500);
        int drawHei = readInt("agvEditorDrawHeight",450);
        int minWid = readInt("agvEditorMinWidth",350);
        int minHei = readInt("agvEditorMinHeight",300);
        editor->setDrawSize(drawWid,drawHei,minWid,minHei);
  }
}


OrderedCltn& AgvSem::getCommands(OrderedCltn& cltn)
{
  cltn.add( *new String(SaveAgvFile));
  cltn.add( *new String(RunAgv));
  cltn.add( *new String(AgvVersion));
  return cltn;
}


IconNode* AgvSem::iconCreate(const String& label,int,int)
{
  if(label == "LineRail")
        return new RailIcon;
  else
  if(label == "TurnRail")
        return new TurnIcon;
  else
  if(label == "ChangeRail")
        return new ChangeIcon;
  else
  if(label == "CartIcon")
        return new CartIcon;
  else
  if(label == "StationIcon")
        return new StationIcon;
  else
        assert(0);
  return NULL;
}


const char* AgvSem::command(const String& label)
{
  if(label == SaveAgvFile)
        return saveAgvFile();
  else
  if(label == RunAgv)
        return runAgv();
  else
  if(label == AgvVersion)
        return editor->raiseVersion("Agv editor version 1.00");

  else
        return NULL;
}


const char* AgvSem::action(IconNode*)
{
  return NULL;
}


const char* AgvSem::saveAgvFile()
{
  OrderedCltn cltn;
  DO(rails,RailIcon,icon)
	cltn.add(*icon->rail());
  OD
  DO(carts,CartIcon,icon)
	cltn.add(*icon->cart());
  OD
  DO(stations,StationIcon,icon)
	cltn.add(*icon->station());
	CompositeIcon* comment = icon->commentIcon();
	icon->station()->setDrawName(comment);
      comment->x(comment->x() + icon->x());
      comment->y(comment->y() + icon->y());
  OD
#if 0
  String fileName = editor->getOnlyName();
  Ostream out(fileName + ".rail");
#else
  Ostream out("layout.rail");
#endif
  cltn.storeOn(out());
  out.close();
  DO(stations,StationIcon,icon)
	icon->station()->resetDrawName();
	CompositeIcon* comment = icon->commentIcon();
      comment->x(comment->x() - icon->x());
      comment->y(comment->y() - icon->y());
  OD
  return NULL;
}


void AgvSem::setPosition(IconNode* icon)
{
  TRACE_FUNC("AgvSem::setPosition");
}


char* AgvSem::checkCollision(IconNode* icon, OrderedCltn& cltn)
{
  if(icon->isKindOf(class_PeriferyIcon))
  {
    if(rails.isEmpty())
	return "Cannot insert a Cart without a Rail";
    if(cltn.size() > 0)
    {
	DO(cltn,IconNode,icon1)
	  if(icon->isA() == icon1->isA())
		return form("This point already contain another %s",
			icon->className());
	OD
    }
  }
  return NULL;
}


char* AgvSem::checkConnect(IconNode* from, IconNode* to, const Class* linkClass, const FloatPoint& start, const FloatPoint& stop)
{
  return NULL;
}


void AgvSem::removeIcon(IconNode* icon)
{
  if(icon->isKindOf(class_RailIcon))
  {
	RailIcon* railIcon = (RailIcon*)icon;
	detach(railIcon);
	Object* ass = railToIcon.removeKey(*railIcon->rail());
	delete ass;
	rails.remove(*railIcon);
  }
  else
  if(icon->isKindOf(class_PeriferyIcon))
  {
	PeriferyIcon* perifIcon = (PeriferyIcon*)icon;
  	if(icon->isKindOf(class_CartIcon))
		carts.remove(*perifIcon);
	else
		stations.remove(*perifIcon);
	perifIcon->removeIcon();

  }
}


void AgvSem::insertIcon(IconNode* icon)
{
  if(icon->isKindOf(class_RailIcon))
  {
	RailIcon* railIcon = (RailIcon*)icon;
	rails.add(*railIcon);
	railToIcon.addAssoc(*railIcon->rail(),*railIcon);
	attach(railIcon);
  }
  else
  if(icon->isKindOf(class_PeriferyIcon))
  {
	PeriferyIcon* perifIcon = (PeriferyIcon*)icon;
  	if(icon->isKindOf(class_CartIcon))
		carts.add(*perifIcon);
	else
		stations.add(*perifIcon);
	RailIcon* railIcon = NULL;
	DO(rails,RailIcon,rail)
	  if(rail->center() == perifIcon->center())
		railIcon = rail;
	OD
	assert(railIcon != NULL);
	perifIcon->addIcon(railIcon);
  }
}


void AgvSem::removeAll()
{
  Semantic::removeAll();
  title = (AgvTitle*)nil;
  rails.removeAll();
  DO(carts,CartIcon,cartIcon)
	cartIcon->removeIcon();
  OD
  carts.removeAll();
  DO(stations,StationIcon,stationIcon)
	stationIcon->removeIcon();
  OD
  stations.removeAll();
  DO(pointToRail,Assoc,ass)
	Object* assInt = ass->value();
	ass->value(*nil);
	delete assInt; // delete AssocInt
  OD
  pointToRail.deleteAll(); // delete Assoc
  railToIcon.deleteAll();  // delete Assoc
  freePoints.deleteAll(); // delete Point;
  // ....
}


bool AgvSem::checkMotion(IconNode* icon, Point& newPnt)
{
  if(icon->isKindOf(class_RailIcon))
  {
    int x = newPnt.x();
    int y = newPnt.y();
    Point minP(x,y);
    double d = MAX_INT;
    DO(freePoints,Point,p)
	double d1 = p->dist(newPnt);
	if(d1 < d)
      {
		d = d1;
		minP = *p;
	}
    OD
    newPnt = minP;
    return BASE::checkMotion(icon,newPnt);
  }
  else
  if(icon->isKindOf(class_PeriferyIcon))
  {
    int x = newPnt.x();
    int y = newPnt.y();
    Point minP(x,y);
    double d = MAX_INT;
    DO(rails,RailIcon,rail)
	double d1 = Point(rail->x(),rail->y()).dist(newPnt);
	if(d1 < d)
	{
		d = d1;
		minP = Point(rail->x(),rail->y());
	}
    OD
    newPnt = minP;
    return BASE::checkMotion(icon,newPnt);
  }
  else
	return BASE::checkMotion(icon,newPnt);
}


void AgvSem::attach(RailIcon* icon)
{
  Point p(icon->x(),icon->y());
  LookupKey* ass = pointToRail.lookupAtKey(p);
  Rail* rail = icon->rail();
  int orient = icon->whereConnect();
  if(ass == nil)
  {
	rail->setPos(RailPos(p.x(), p.y(), 0));
	orient = 0;
  }
  else
  {
    AssocInt* ass1 = CASTP(AssocInt,ass->value());
    Rail* lastRail = CASTP(Rail,ass1->key());
    int lastOrient = ass1->intValue();
    lastRail->attachIn(lastOrient,rail,orient);
    Object* remKey = pointToRail.removeKey(p);
    assert(ass ==  remKey);
    Object* pt = freePoints.remove(p);
    assert(pt == ass->key());
    delete ass;
    delete pt;
    delete ass1;
  }
  rail->x(-icon->x() + 0.5);
  rail->y(-icon->y() + 0.5);
  icon->compositeIcon()->add(*rail);
  icon->movable(NO);
  for(int i = 1; i <= rail->railCapacity(); i++)
  {
    if(i == orient)
	continue;
    RailPos& pos = rail->posAt(i);
    Point tmp((int)pos.x(), (int)pos.y());
    ass = pointToRail.lookupAtKey(tmp);
    if(ass == nil)
    {
      Point* p = new Point(tmp);
      AssocInt* assI = new AssocInt(*rail,i);
      pointToRail.addAssoc(*p,*assI);
      freePoints.add(*p);
    }
    else
    {
      AssocInt* ass1 = CASTP(AssocInt,ass->value());
      Rail* lastRail = CASTP(Rail,ass1->key());
      int lastOrient = ass1->intValue();
      lastRail->attachIn(lastOrient,rail,i);
      Object* remKey = pointToRail.removeKey(tmp);
      assert(ass ==  remKey);
      Object* pt = freePoints.remove(tmp);
      assert(pt == ass->key());
      delete ass;
      delete pt;
      delete ass1;
    }
  }
}


void AgvSem::detach(RailIcon* icon)
{
  Rail* rail = icon->rail();
  for(int i = 0; i++ < rail->railCapacity();)
  {
	int attachOrient = 1;
	Rail* attachRail = rail->detachAt(i,attachOrient);
	if(attachRail == nil)
	{
	  Point tmp((int)rail->posAt(i).x(),
		 (int)rail->posAt(i).y());
    	  LookupKey* ass = pointToRail.removeKey(tmp);
    	  assert(ass != nil);
      	  AssocInt* ass1 = CASTP(AssocInt,ass->value());
     	  Object* pt = freePoints.remove(tmp);
     	  assert(pt == ass->key());
      	  delete ass;
     	  delete pt;
      	  delete ass1;
	}
	else
	{
	  Point* p = new Point(
		(int)attachRail->posAt(attachOrient).x(),
		(int)attachRail->posAt(attachOrient).y());
	  assert(! pointToRail.includes(*p) );
	  AssocInt* assI = new AssocInt(*attachRail,attachOrient);
  	  pointToRail.addAssoc(*p,*assI);
  	  freePoints.add(*p);
	}
  }
}


RailIcon* AgvSem::iconAtRail(Rail* rail)
{
  return CASTP(RailIcon,railToIcon.atKey(*rail));
}


const char* AgvSem::runAgv()
{

	static Rail* lastRail = NULL;
	static Rail* rail = NULL;

  Cart* agv = new Cart;
  OrderedCltn changes;
  DO(rails,RailIcon,icon)
	if(icon->rail()->isKindOf(class_ChangeRail))
		changes.add(*icon->rail());
      icon->compositeIcon()->remove(*icon->selectIcon());
  OD
  editor->refresh();
  flushXwindow();
  int changeIndex = 0;
  bool changePresent = changes.size() > 0;

  if(!validObject(lastRail) || !validObject(rail) 
	|| !railToIcon.includesKey(*rail))
  {

      lastRail = CASTP(RailIcon,rails.first())->rail();
      rail = lastRail->nextRail();
  }

  IconNode* icon = iconAtRail(rail);
  rail->setFrom(lastRail);
  rail->add(*agv->setIcon(rail));
      
  for(int i = 0; i < 200; i++)
  {
	sleepForTime(150);
	// flushXwindow();
	editor->eraseIcon(icon);
	rail->remove(*agv->getIcon());
	editor->drawIcon(icon);
	lastRail = rail;
	rail = rail->nextRail();
	icon = iconAtRail(rail);
	editor->eraseIcon(icon);
  	rail->setFrom(lastRail);
  	rail->add(*agv->setIcon(rail));
	editor->drawIcon(icon);

	if(changePresent && i % 3 == 0)
      {
	  CASTP(ChangeRail,changes.atIndex(changeIndex++))
		->toggleChange();
	  changeIndex = changeIndex % changes.size();
      }
  }
  editor->eraseIcon(icon);
  rail->remove(*agv->getIcon());
  editor->drawIcon(icon);
  DO(rails,RailIcon,icon)
     icon->compositeIcon()->add(*icon->selectIcon());
  OD
  editor->refresh();
  delete agv;
  return NULL;
}





Object* AgvSem::editorTrace(String& s)
{
  if(s == "title")
  {
	s = title->printOnString();
	return (Object*)title;
  }
  else
  if(s == "railToIcon")
  {
	s = railToIcon.printOnString();
	return (Object*)&railToIcon;
  }
  else
  if(s == "rails")
  {
	s = rails.printOnString();
	return (Object*)&rails;
  }
  else
  if(s == "carts")
  {
	s = carts.printOnString();
	return (Object*)&carts;
  }
  else
  if(s == "stations")
  {
	s = stations.printOnString();
	return (Object*)&stations;
  }
  else
  if(s == "freePoints")
  {
	s = freePoints.printOnString();
	return (Object*)&freePoints;
  }
  else
  if(s == "pointToRail")
  {
	s = pointToRail.printOnString();
	return (Object*)&pointToRail;
  }
  return nil;
}

/* End of File AGVSEM */
