/* File Cart.C  implement ....*/
#define CART_C


#include <oak/oakstrn.h>
#include <oak/symbol.h>
#include <oak/cbacks.h>

#include "Cart.h"

#include "Rail.h"
#include <oak/dictiona.h>

#include <oak/lookupke.h>

Dictionary unUsedIcons;

Symbol CartEmpty = "CartEmpty";
Symbol CartFull = "CartFull";
Symbol CartGoHome = "CartGoHome";

#define DELETE_ICON 1


DEFINE_SIMPLE_CLASS(Cart,RailPerifery,1,NULL);
#define THIS Cart
#define BASE RailPerifery


void Cart::storer(fileDescTy& fd) const
  { BASE::storer(fd); }

void Cart::storer(ostream& strm) const
{
  TRACE_FUNC("Cart::storer");
  BASE::storer(strm);
}

Cart::Cart(fileDescTy& fd, Cart& where)
  :BASE(fd,where)  { THIS_EQUAL_WHERE(where); } 

Cart::Cart(istream& strm, Cart& where)
  :BASE(strm,where)
{
  THIS_EQUAL_WHERE(where);
  TRACE_FUNC("Cart::fileConstructor");
  state = CartEmpty;
  initIcon();
}


Cart::Cart(const String& name)
  : BASE(name)
{
  TRACE_FUNC("Cart::Cart");
  state = CartEmpty;
  initIcon();
}


Cart::Cart()
  : BASE("Cart")
{
  TRACE_FUNC("Cart::Cart");
  state = CartEmpty;
  initIcon();
}



Cart::~Cart()
{
  TRACE_FUNC("Cart::~Cart");
#if DELETE_ICON
  icon->deleteAll();
  delete icon;
#endif
}





CompositeIcon& Cart::createIcon()
{
  TRACE_FUNC("Cart::createIcon");
  icon = new CompositeIcon;
  double width = 3.0, height = 1.0;
  double wi2 = width/2, he2 = height/2;

  double c = cos(icon_pos.radAngle());
  double s = sin(icon_pos.radAngle());

  double w = width * c;
  double h = width * s;
  double x = icon_pos.x() - wi2 * c + he2 * s;
  double y = icon_pos.y() - he2 * c - wi2 * s;

  icon->add(*new DrawLine(x,y,w,h));
  x -= height * s;
  y += height * c;
  icon->add(*new DrawLine(x,y,w,h));

  w = - height * s;
  h = height * c;
  x = icon_pos.x() - wi2 * c + he2 * s;
  y = icon_pos.y() - he2 * c - wi2 * s;
  icon->add(*new DrawLine(x,y,w,h));
  x += width * c;
  y += width * s;
  icon->add(*new DrawLine(x,y,w,h));
  return *icon;
}


void Cart::initIcon()
{
  TRACE_FUNC("Cart::initIcon");
#if DELETE_ICON
  createIcon();
#else
  LookupKey* ass = unUsedIcons.lookupAtKey(icon_pos);
  if(ass == nil)
  {
	unUsedIcons.addAssoc(*new RailPos(icon_pos),
	createIcon());
  }
  else
  {
      icon = CASTP(CompositeIcon,ass->value());
  }
#endif
}


void Cart::setColor()
{
  TRACE_FUNC("Cart::setColor");
  if(state == CartEmpty)
	icon->foreground("green");
  else
  if(state == CartFull)
	icon->foreground("red");
  else 
  if(state == CartGoHome)
	icon->foreground("yellow");
  else 
	assert(0);
}



CompositeIcon* Cart::setIcon(Rail* rail)
{
  TRACE_FUNC("Cart::getIcon(const RailPos&)");
  rail_at = rail;
  RailPos pos = rail->getCartPos();
  if(pos != icon_pos)
  {
#if DELETE_ICON
    icon->deleteAll();
    delete icon;
    icon_pos = pos;
    createIcon();
#else
    if(!unUsedIcons.includesKey(icon_pos))
    {
	unUsedIcons.addAssoc(*new RailPos(icon_pos),
		*icon);
    }
    icon_pos = pos;
    initIcon();
#endif
  }

  setColor();
  return icon;
}


void Cart::setStatus(const Symbol& status)
{
  TRACE_FUNC("Cart::setStatus");
  BASE::setStatus(status);
  setColor();
}




Object* Cart::editorTrace(String& )
{
  return nil;
}

/* End of File Cart.C */

