/* File RailPos.C  implement ....*/
#define RAILPOS_C


#include <oak/oakstrn.h>
#include <oak/symbol.h>
#include <oak/cbacks.h>

#include "RailPos.h"


double mod180(double ang)
{
  if(ang > 180)
	ang -= 360;
  else
  if(ang <= -180)
	ang += 360;

  return ang;
}

double mod360(double ang)
{
  if(ang >= 360)
	ang -= 360;
  else
  if(ang < 0)
	ang += 360;

  return ang;
}



DEFINE_SIMPLE_CLASS(RailPos,Object,1,NULL);
#define THIS RailPos
#define BASE Object


void RailPos::storer(fileDescTy& fd) const
  { BASE::storer(fd); }

void RailPos::storer(ostream& strm) const
{
  TRACE_FUNC("RailPos::storer");
  BASE::storer(strm);
  strm << " " << xc << " " << yc << " " << ang << " ";
}

RailPos::RailPos(fileDescTy& fd, RailPos& where)
  :BASE(fd,where)  { THIS_EQUAL_WHERE(where); } 

RailPos::RailPos(istream& strm, RailPos& where)
  :BASE(strm,where)
{
  THIS_EQUAL_WHERE(where);
  strm >> xc >> yc >> ang;
}


RailPos::RailPos(double x, double y, double angle)
  : xc(x), yc(y), ang(mod180(angle))
{
}


RailPos::RailPos(const RailPos& pos)
  : xc(pos.xc), yc(pos.yc), ang(pos.ang)
{
  TRACE_FUNC("RailPos::RailPos");
}







double RailPos::x() const
{
  return xc;
}


double RailPos::y() const
{
  return yc;
}


double RailPos::angle() const
{
  return ang;
}


double RailPos::radAngle() const
{
  return TO_RAD(ang);
}


void RailPos::operator=(const RailPos& pos)
{
  TRACE_FUNC("RailPos::operator=");
  xc = pos.xc;
  yc = pos.yc;
  ang = pos.ang;
}


RailPos RailPos::operator+(const RailPos& pos) const
{
  double c = cos(TO_RAD(ang));
  double s = sin(TO_RAD(ang));
  return RailPos(xc + pos.xc * c - pos.yc * s,
                 yc + pos.yc * c + pos.xc * s,
                 ang + pos.ang);
}


RailPos RailPos::operator-(const RailPos& pos) const
{
  return RailPos(xc - pos.xc, yc - pos.yc,
				ang - pos.ang);
}


RailPos RailPos::operator-() const
{
  return RailPos(xc, yc, ang - 180);
}


void RailPos::printOn(ostream& strm) const
{
  strm << className() << "[" << xc
			<< "," << yc
			<< "," << ang << "]";
}


bool RailPos::isEqual(const Object& ob) const
{
  TRACE_FUNC("RailPos::isEqual");
  if(!ob.isKindOf(class_RailPos))
	return NO;
  RailPos& pos = *(RailPos*)&ob;
  return xc == pos.xc && yc == pos.yc && ang == pos.ang;
}


size_t RailPos::hash() const
{
  TRACE_FUNC("RailPos::hash");
  return bytehash(&xc, 3 * sizeof(float));
}




Object* RailPos::editorTrace(String& )
{
  return nil;
}

/* End of File RailPos.C */

