/* File RailSemaph.C  implement ....*/
#define RAILSEMAPH_C


#include <oak/oakstrn.h>
#include <oak/symbol.h>
#include <oak/cbacks.h>

#include "RailSemaph.h"

#include "ChangeRail.h"
#include <mondrian/fillarc.h>


DEFINE_SIMPLE_CLASS(RailSemaph,Object,1,NULL);
#define THIS RailSemaph
#define BASE Object


void RailSemaph::storer(fileDescTy& fd) const
  { BASE::storer(fd); }

void RailSemaph::storer(ostream& strm) const
{
  TRACE_FUNC("RailSemaph::storer");
  BASE::storer(strm);
  change1->storeOn(strm);
  change2->storeOn(strm);
  light1->storeOn(strm);
  light2->storeOn(strm);
  strm << " " << busy << " ";
}

RailSemaph::RailSemaph(fileDescTy& fd, RailSemaph& where)
  :BASE(fd,where)  { THIS_EQUAL_WHERE(where); } 

RailSemaph::RailSemaph(istream& strm, RailSemaph& where)
  :BASE(strm,where),sem(1)
{
  THIS_EQUAL_WHERE(where);
  TRACE_FUNC("RailSemaph::fileConstructor");
  change1 = (ChangeRail*)readFrom(strm);
  change2 = (ChangeRail*)readFrom(strm);
  light1 = (FillArc*)readFrom(strm);
  light2 = (FillArc*)readFrom(strm);
  strm >> busy;
  if(busy)
	sem.wait();	
}


RailSemaph::RailSemaph()
  : sem(1)
{
  TRACE_FUNC("RailSemaph::RailSemaph");
  change1 = (ChangeRail*)nil;
  change2 = (ChangeRail*)nil;
  light1 = (FillArc*)nil;
  light2 = (FillArc*)nil;
  busy = NO;
}



RailSemaph::~RailSemaph()
{
  TRACE_FUNC("RailSemaph::~RailSemaph");
  if(change1 != nil)
	change1->removeSemaph(this,light1);
  if(change2 != nil)
	change2->removeSemaph(this,light2);
}






void RailSemaph::setBusy(bool value)
{
  TRACE_FUNC("RailSemaph::setBusy");
  busy = value;
  String color = "green";
  if(busy)
	color = "red";

  if(light1 != nil)
	light1->foreground(color);
  if(light2 != nil)
  	light2->foreground(color);
}


bool RailSemaph::isBusy()
{
  TRACE_FUNC("RailSemaph::isBusy");
  return busy;
}


void RailSemaph::addChange(ChangeRail* rail, int where)
{
  TRACE_FUNC("RailSemaph::addChange");
  if(change1 == nil)
  {
  	change1 = rail;
	light1 = rail->addSemaph(this,where);
  }
  else
  {
	assert(change2 == nil);
  	change2 = rail;
	light2 = rail->addSemaph(this,where);
  }
}


ChangeRail* RailSemaph::otherChange(ChangeRail* rail)
{
  TRACE_FUNC("RailSemaph::otherChange");
  if(change1 == rail)
	return change2;
  else
  {
	assert(change2 == rail);
	return change1;
  }
}


void RailSemaph::removeChange(ChangeRail* rail)
{
  TRACE_FUNC("RailSemaph::removeChange");
  if(change1 == rail)
  {
	change1->removeSemaph(this,light1);
	change1 = (ChangeRail*)nil;
	light1 = (FillArc*)nil;
  }
  else
  {
	assert(change2 == rail);
	change2->removeSemaph(this,light2);
	change2 = (ChangeRail*)nil;
	light2 = (FillArc*)nil;
  }
}


void RailSemaph::wait()
{
  TRACE_FUNC("RailSemaph::wait");
  sem.wait();
  setBusy();
}


void RailSemaph::signal()
{
  TRACE_FUNC("RailSemaph::signal");
  sem.signal();
  setBusy(NO);
}




Object* RailSemaph::editorTrace(String& s)
{
  if(s == "change2")
  {
	s = change2->printOnString();
	return (Object*)change2;
  }
  else
  if(s == "change1")
  {
	s = change1->printOnString();
	return (Object*)change1;
  }
  else
  if(s == "light1")
  {
	s = light1->printOnString();
	return (Object*)light1;
  }
  else
  if(s == "light2")
  {
	s = light2->printOnString();
	return (Object*)light2;
  }
  else
  if(s == "sem")
  {
	s = sem.printOnString();
	return (Object*)&sem;
  }
  return nil;
}

/* End of File RailSemaph.C */

