/* File Station.C  implement ....*/
#define STATION_C


#include <oak/oakstrn.h>
#include <oak/symbol.h>
#include <oak/cbacks.h>

#include "Station.h"

#include <mondrian/compicon.h>
#include "Rail.h"

Symbol StationFree = "StationFree";
Symbol StationBusy = "StationBusy";



DEFINE_SIMPLE_CLASS(Station,RailPerifery,1,NULL);
#define THIS Station
#define BASE RailPerifery


void Station::storer(fileDescTy& fd) const
  { BASE::storer(fd); }

void Station::storer(ostream& strm) const
{
  TRACE_FUNC("Station::storer");
  BASE::storer(strm);
  draw_name->storeOn(strm);
}

Station::Station(fileDescTy& fd, Station& where)
  :BASE(fd,where)  { THIS_EQUAL_WHERE(where); } 

Station::Station(istream& strm, Station& where)
  :BASE(strm,where)
{
  THIS_EQUAL_WHERE(where);
  TRACE_FUNC("Station::fileConstructor");
  draw_name = (CompositeIcon*)readFrom(strm);
  createIcon();
}


Station::Station(const String& name)
  : BASE(name)
{
  TRACE_FUNC("Station::Station");
  draw_name = (CompositeIcon*)nil;
  createIcon();
}


Station::Station()
  : BASE("Station")
{
  TRACE_FUNC("Station::Station");
  draw_name = (CompositeIcon*)nil;
  createIcon();
}



Station::~Station()
{
  TRACE_FUNC("Station::~Station");
}





CompositeIcon& Station::createIcon()
{
  TRACE_FUNC("Station::createIcon");
  icon = new CompositeIcon;
  double width = 3.0, height = 1.0;
  double wi2 = width/2, he2 = height/2;

  double c = cos(icon_pos.radAngle());
  double s = sin(icon_pos.radAngle());

  double w = width * c;
  double h = width * s;
  double x = icon_pos.x() - wi2 * c + he2 * s;
  double y = icon_pos.y() - he2 * c - wi2 * s;

  icon->add(*new DrawLine(x,y,w,h));
  x -= height * s;
  y += height * c;
  icon->add(*new DrawLine(x,y,w,h));
  w = w + height * s;
  h = h - height * c;
  icon->add(*new DrawLine(x,y,w,h));

  w = - height * s;
  h = height * c;
  x = icon_pos.x() + wi2 * c + he2 * s;
  y = icon_pos.y() - he2 * c + wi2 * s;
  icon->add(*new DrawLine(x,y,w,h));
  x -= width * c;
  y -= width * s;
  icon->add(*new DrawLine(x,y,w,h));

  w = w + width * c;
  h = h + width * s;
  icon->add(*new DrawLine(x,y,w,h));
  if(draw_name != nil)
	icon->add(*draw_name);
  return *icon;
}



CompositeIcon* Station::setIcon(Rail* rail)
{
  TRACE_FUNC("Station::getIcon(const RailPos&)");
  RailPos pos = rail->getStationPos();
  rail_at = rail;
  if(pos != icon_pos)
  {
	icon_pos = pos;
	delete icon;
    	createIcon();
  }
  return icon;
}


void Station::setDrawName(CompositeIcon* drawStr)
{
  TRACE_FUNC("Station::setDrawName");
  draw_name = drawStr;
}


void Station::resetDrawName()
{
  TRACE_FUNC("Station::resetDrawName");
  draw_name = (CompositeIcon*)nil;
}




Object* Station::editorTrace(String& s)
{
  if(s == "draw_name")
  {
	s = draw_name->printOnString();
	return (Object*)draw_name;
  }
  return nil;
}

/* End of File Station.C */

