/* File TurnRail.C  implement ....*/
#define TURNRAIL_C


#include <oak/oakstrn.h>
#include <oak/symbol.h>
#include <oak/cbacks.h>

#include "TurnRail.h"



DEFINE_SIMPLE_CLASS(TurnRail,Rail,1,NULL);
#define THIS TurnRail
#define BASE Rail


void TurnRail::storer(fileDescTy& fd) const
  { BASE::storer(fd); }

void TurnRail::storer(ostream& strm) const
{
  TRACE_FUNC("TurnRail::storer");
  BASE::storer(strm);
  strm << " " << rail_ang << " ";
}

TurnRail::TurnRail(fileDescTy& fd, TurnRail& where)
  :BASE(fd,where)  { THIS_EQUAL_WHERE(where); } 

TurnRail::TurnRail(istream& strm, TurnRail& where)
  :BASE(strm,where)
{
  THIS_EQUAL_WHERE(where);
  strm >> rail_ang;
}


TurnRail::TurnRail(double ang, double ray)
  :BASE(ray), rail_ang(ang)
{
  assert(ang > -180 && ang < 180);
}






void TurnRail::setGeometry(const RailPos& pos, int orient)
{
  assert(orient > 0 && orient < 3);

  double ang = rail_ang;
  double radAng = TO_RAD(rail_ang);
  double ray = rail_len;
  double dx = ray * sin(radAng);
  double dy = ray * (1 - cos(radAng));
  posCltn[orient - 1] = new RailPos(-pos);
  if(orient == 1)
    posCltn[1] = new RailPos(pos + RailPos(dx,dy,ang));
  else
    posCltn[0] = new RailPos(pos+ RailPos(dx,-dy,-ang));

}


void TurnRail::createIcon()
{
  TRACE_FUNC("TurnRail::createIcon");

  double ang = rail_ang;
  double radAng = TO_RAD(rail_ang);
  double ray = rail_len;
  double dx = ray * sin(radAng);
  double dy = ray * (1 - cos(radAng));

  RailPos& pos = posAt(1);
  double startAng = pos.angle();
  double stRadAng = pos.radAngle();
  double x = pos.x() - ray + ray * sin(stRadAng);
  double y = pos.y() - ray - ray * cos(stRadAng);
  
  double w = 2 * ray;
  
  if(ray < 0)
	startAng -= 180;
  add(*new DrawArc(x,y,w,w, -90 - startAng, -ang));
}


RailPos TurnRail::getConveyPos()
{
  RailPos& p1 = posAt(1);
  RailPos& p2 = posAt(2);
  double ang1 = (-p1).angle();
  double ang2 = p2.angle();
  double ang = ang1 + ang2;
  if(ABS(ang1 - ang2) > 180)
	ang = mod360(ang1) + mod360(ang2);
  ang /= 2;

  double x = ( p1.x() + p2.x() ) / 2;
  double y = ( p1.y() + p2.y() ) / 2;
  double r = ray() * (1 - cos(TO_RAD(rail_ang / 2)));
  x += r * sin(TO_RAD(ang));
  y -= r * cos(TO_RAD(ang));
  
  return RailPos(x, y, ang);
}



double TurnRail::ray()
{
  return rail_len;
}




Object* TurnRail::editorTrace(String& )
{
  return nil;
}

/* End of File TurnRail.C */

