/* File topLevel-532.C  implementation of class TopLevel */

/*		Copyright (c) 1990 SYCO			  */
/*		  All Rights Reserved			  */
/*							  */
/*  THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF SYCO   */
/*  The copyright notice above does not evidence any      */
/*  actual or intended publication of such source code.   */


#define TOPLEVEL_532_C

#include <oak/oakstrn.h>
#include <oak/symbol.h>
#include <oak/cbacks.h>

#include "topLevel-532.h"

#include <oak/orderedc.h>


Symbol FORK = "FORK";
Symbol ADD_ORDERS = "ADD_ORDERS";
Symbol REMOVE_ORDERS = "REMOVE_ORDERS";


#include <oak/oakstrm.h> 
#include <oak/clock.h>
#if 0
#include <rodin/oakproc.h> 
#endif
bool orderPlannedFunc(const Cback_struct& cb) 
 { 
   cout << "Callback raised by owner " << *cb.owner() << endl; 
   cout << "Callback name = " << cb.name() << endl; 
   cout << "User Data = " << *cb.userData() << endl; 
   cout << "Call Data = " << *cb.callData() << endl; 
   return YES; 
 }
 
bool vseqCltnFunc(const Cback_struct& cb) 
{ 
   cout << "Callback name = " << cb.name() << endl; 
   cout << "Callback raised by owner " << cb.owner()->className() 
        << endl; 
   if(cb.callData() != NULL) 
       cout << "Call Data = " << *cb.callData() << endl; 
   return YES; 
}
main(int,char**) 
{ 
   setRealTime(100); // the resolution is 100 msec.
   TopLevel* topLevel = new TopLevel; 
   topLevel->doIt();
   runCBTime(); 
   delete topLevel; 
   return 0; 
}

DEFINE_SIMPLE_CLASS(TopLevel,Object,1,NULL);
#define THIS TopLevel
#define BASE Object


TopLevel::TopLevel()
  : orders(class_OrderedCltn) 
    // the constructor of orders is invoked, 
    // an OrderedCltn is passed 

{
  TRACE_FUNC("TopLevel::TopLevel");
   orders.addCback(AnyCback,vseqCltnFunc); 
     // any event is monitored 

}







void TopLevel::insertError()
{
  TRACE_FUNC("TopLevel::insertError");
  Order automaticOrder("OrderA"); 
  orders.add(automaticOrder); 
  cout << "All orders are " << orders << endl; 
  // forgot to remove automaticOrder 

}


void TopLevel::forkSecondProcess(const Cback& cb)
{
  TRACE_FUNC("TopLevel::forkSecondProcess");
    
     cout << "Active Process is the second process" << endl;  
  
     attachTo(&orders,AddSignal,&THIS::answerToEvents);  
     attachTo(&orders,RemoveSignal,&THIS::answerToEvents);  
  
     cout << "SecondProcess waits for AddSignal" << endl;  
}


void TopLevel::answerToEvents(const Cback& cb)
{
  TRACE_FUNC("TopLevel::answerToEvents");
  if(cb.name() == AddSignal)
  {
	  attachTo(this,REMOVE_ORDERS,&THIS::removeOrders);
	  callDelayedCbacks(10,REMOVE_ORDERS);
  
   }  
  if(cb.name() == RemoveSignal)  
  { 
          cout << "The state of all present processes is" << endl;  
          detachFrom(&orders,AddSignal,&THIS::answerToEvents);  
          detachFrom(&orders,RemoveSignal,&THIS::answerToEvents);  
          cout << "SecondProcess Terminates Now" << endl;  
  }
}


void TopLevel::addOrders(const Cback& cb)
{
  TRACE_FUNC("TopLevel::addOrders");

  cout << "Main process suspends itself and SecondProcess starts"  
    << endl;
  if(CASTP(Integer,cb.callData())->value() < currentCBTime())
	cout << "WARNING REAL TIME NOT RESPECTED " <<
	"Current time should have been = " 
	<< CASTP(Integer,cb.callData())->value() << endl;

  cout << "Current time = " << currentCBTime() << endl; 
  orders.add(*new Order("Order1"));  
  detachFrom(this,ADD_ORDERS,&THIS::addOrders);
}


void TopLevel::removeOrders(const Cback& cb)
{
  TRACE_FUNC("TopLevel::removeOrders");

  Order* order = CASTP(Order,orders.removeFirst());  
  cout << "the Order removed is " << *order << endl;  
  cout << "Current Time = " << currentCBTime() << endl;
  delete order;
  detachFrom(this,REMOVE_ORDERS,&THIS::removeOrders);
  cout << "Main Process runs again after SecondProcess Termination"  
       << endl;  
  breakCBTime();
}



void TopLevel::doIt()
{
  TRACE_FUNC("TopLevel::doIt");
//  int mainPriority = mainProcess->priority();  
  
//  Process* secondProcess = new Process("SecondProcess",  

  // secondProcess has a greater priority than mainProcess  
  
  cout << "Main Process before a FORK of the SecondProcess" << endl;  
  cout << "Current time = " << currentCBTime() << endl;
  attachTo(this,FORK,&THIS::forkSecondProcess);
  attachTo(this,ADD_ORDERS,&THIS::addOrders);
  callCbacks(FORK);
 
  cout << "Main Process continues after SecondProcess waitEvent"  
       << endl;  
  cout << "Active Process is the first process" << endl;  

  callDelayedCbacks(5,ADD_ORDERS,new Integer(currentCBTime() + 5)); 
	// the next time is passed as a callData to check time consistency
 
  attachTo(this,FORK,&THIS::forkSecondProcess);
  cout << "Some time is being  wasted ..." << endl;
  for(int i =0;i<1000000;i++) // some time is wasted
	;
  cout << "Some time is gone... current time = " 
	<< currentCBTime() << endl;
 
}




Object* TopLevel::editorTrace(String& s)
{
  if(s == "orders")
  {
	s = orders.printOnString();
	return (Object*)&orders;
  }
  return nil;
}

/* End of File TOPLEVEL_532 */
