/* File topLevel-63.C  implement ....*/
#define TOPLEVEL_63_C


#include <oak/oakstrn.h>
#include <oak/symbol.h>
#include <oak/cbacks.h>

#include "topLevel-63.h"

#include <mondrian/form.h>
#include <mondrian/box.h>
#include <mondrian/graphic.h>
#include <mondrian/drawicon.h>
#include <mondrian/pushbutt.h>
#include <mondrian/drawicon.h>
#include <mondrian/graphic.h>

Symbol DoInsert = "DoInsert"; 
Symbol DoMove = "DoMove"; 

#define insertButton  CASTP(CppPushButton,buttons[0]) 
#define moveButton  CASTP(CppPushButton,buttons[1]) 
#define quitButton  CASTP(CppCommand,buttons[2]) 

bool setToInsertFunc(const Cback_struct& cb) 
{ 
   CASTP(TopLevel,cb.userData())->setToInsert(); 
   return YES; 
}
 
bool setToMoveFunc(const Cback_struct& cb) 
{ 
   CASTP_USERDATA(TopLevel,toplevel);
   toplevel->setToMove(); 
   return YES; 
} 
 
bool graphDownFunc(const Cback_struct& cb) 
{ 
   WorldCursor* cur = (WorldCursor*)cb.callData(); 
   CASTP_USERDATA(TopLevel,toplevel);
   toplevel->graphDown(cur); 
   return YES; 
} 
 
bool graphMoveFunc(const Cback_struct& cb) 
{ 
   WorldCursor* cur = (WorldCursor*)cb.callData(); 
   CASTP_USERDATA(TopLevel,toplevel);
   toplevel->graphMove(cur); 
   return YES; 
} 
 
bool graphUpFunc(const Cback_struct& cb) 
{ 
   WorldCursor* cur = (WorldCursor*)cb.callData(); 
   CASTP_USERDATA(TopLevel,toplevel);
   toplevel->graphUp(cur); 
   return YES; 
} 
 
bool quitFunc(const Cback_struct&) 
{ 
   exitMainLoop(); 
   return YES; 
} 
 
main(int,char**) 
{ 
  TopLevel* toplevel = new TopLevel; 
  toplevel->doIt(); 
  delete toplevel; 
  return 0; 
} 

DEFINE_SIMPLE_CLASS(TopLevel,Object,1,NULL);
#define THIS TopLevel
#define BASE Object


TopLevel::TopLevel()
{
  TRACE_FUNC("TopLevel::TopLevel");
   base = new CppForm(topLevelWidget,"base"); 
   box = new CppBox(base,"box"); 
   graph = new CppGraphic(base,"graph"); 
   base->leftRight(box,graph); 
   graph->addCback(DownGraphic,graphDownFunc,this); 
   graph->addCback(MoveDownGraphic,graphMoveFunc,this); 
   graph->addCback(UpGraphic,graphUpFunc,this); 
 
   CppPushButton* button = new CppPushButton(box,"Insert"); 
   button->addCback(CommandPress,setToInsertFunc,this); 
   buttons.add(*button); 
   button->set(); 
   state = DoInsert; 
 
   button = new CppPushButton(box,"Move"); 
   button->addCback(CommandPress,setToMoveFunc,this); 
   buttons.add(*button); 
 
   CppCommand* quit = new CppCommand(box,"Quit"); 
   quit->addCback(CommandPress,quitFunc,this); 
   buttons.add(*quit); 
 
   moveObject = NULL; 

}



TopLevel::~TopLevel()
{
  TRACE_FUNC("TopLevel::~TopLevel");
  deleteWidget(base);
}





void TopLevel::setToInsert()
{
  TRACE_FUNC("TopLevel::setToInsert");
  if(state == DoMove) 
        moveButton->reset(); 
 
  insertButton->reset(); 
  state = DoInsert; 
}


void TopLevel::setToMove()
{
  TRACE_FUNC("TopLevel::setToMove");
  if(state == DoInsert) 
        insertButton->reset(); 
 
  moveButton->reset(); 
  state = DoMove; 
}


void TopLevel::graphDown(WorldCursor* cur)
{
  TRACE_FUNC("TopLevel::graphDown");
  if(state == DoInsert) 
  { 
    DrawRect* rect = new DrawRect(cur->x,cur->y,100,100); 
    rect->foreground(graph->readString("myDefaultColor","green")); 
    graph->add(*rect); 
  } 
  else 
  { 
    DrawIcon* icon = graph->iconAt(*cur); 
    if(icon == nil) 
        return; 
    icon->highLight(graph->readString("myDefaultHighColor","white")); 
    graph->draw(icon); 
    moveObject = icon; 
  } 

}


void TopLevel::graphMove(WorldCursor* cur)
{
  TRACE_FUNC("TopLevel::graphMove");
  if(state != DoMove || moveObject == NULL) 
        return; 
 
  graph->erase(moveObject); 
  moveObject->moveTo(cur->x,cur->y); 
  graph->redraw(); // redraw all icon. 
}


void TopLevel::graphUp(WorldCursor* cur)
{
  TRACE_FUNC("TopLevel::graphUp");
  if(state != DoMove || moveObject == NULL) 
        return; 
 
  graph->erase(moveObject); 
  moveObject->moveTo(cur->x,cur->y); 
  graph->redraw(); // redraw all icons. 
  moveObject = NULL; 
}



void TopLevel::doIt()
{
  TRACE_FUNC("TopLevel::doIt");
  baseMainLoop();
}




Object* TopLevel::editorTrace(String& s)
{
  if(s == "box")
  {
	s = box->printOnString();
	return (Object*)box;
  }
  else
  if(s == "base")
  {
	s = base->printOnString();
	return (Object*)base;
  }
  else
  if(s == "graph")
  {
	s = graph->printOnString();
	return (Object*)graph;
  }
  else
  if(s == "state")
  {
	s = state.printOnString();
	return (Object*)&state;
  }
  else
  if(s == "moveObject")
  {
	s = moveObject->printOnString();
	return (Object*)moveObject;
  }
  else
  if(s == "buttons")
  {
	s = buttons.printOnString();
	return (Object*)&buttons;
  }
  return nil;
}

/* End of File topLevel-63.C */

